<?php
namespace Service\Manager\Mongo\Query;


class Expr {
	protected $query = array();
	protected $currentField;

	public function addAnd($expression) {
		$this->query['$and'][] = $expression instanceof Expr ? $expression->getQuery() : $expression;
		return $this;
	}

	public function addNor($expression) {
		$this->query['$nor'][] = $expression instanceof Expr ? $expression->getQuery() : $expression;
		return $this;
	}

	public function addOr($expression) {
		$this->query['$or'][] = $expression instanceof Expr ? $expression->getQuery() : $expression;
		return $this;
	}

	public function all(array $values) {
		return $this->operator('$all', (array)$values);
	}

	public function each(array $values) {
		return $this->operator('$each', $values);
	}

	public function elemMatch($expression) {
		return $this->operator('$elemMatch', $expression instanceof Expr ? $expression->getQuery() : $expression);
	}

	public function equals($value) {
		if ($this->currentField) {
			$this->query[$this->currentField] = $value;
		} else {
			$this->query = $value;
		}
		return $this;
	}

	public function exists($bool) {
		return $this->operator('$exists', (boolean)$bool);
	}

	public function field($field) {
		$this->currentField = (string)$field;
		return $this;
	}

	public function getCurrentField() {
		return $this->currentField;
	}

	public function getQuery() {
		return $this->query;
	}


	public function setQuery(array $query) {
		$this->query = $query;
	}


	public function gt($value) {
		return $this->operator('$gt', $value);
	}

	public function gte($value) {
		return $this->operator('$gte', $value);
	}


	public function in(array $values) {
		return $this->operator('$in', array_values($values));
	}

    public function nin(array $values) {
        return $this->operator('$nin', array_values($values));
    }


	public function lt($value) {
		return $this->operator('$lt', $value);
	}


	public function lte($value) {
		return $this->operator('$lte', $value);
	}


	public function mod($divisor, $remainder = 0) {
		return $this->operator('$mod', array($divisor, $remainder));
	}


	public function not($expression) {
		return $this->operator('$not', $expression instanceof Expr ? $expression->getQuery() : $expression);
	}


	public function notEqual($value) {
		return $this->operator('$ne', $value);
	}

	public function notIn(array $values) {
		return $this->operator('$nin', array_values($values));
	}


	public function operator($operator, $value) {
		$this->wrapEqualityCriteria();

		if ($this->currentField) {
			$this->query[$this->currentField][$operator] = $value;
		} else {
			$this->query[$operator] = $value;
		}
		return $this;
	}

	public function range($start, $end) {
		return $this->operator('$gte', $start)->operator('$lte', $end);
	}

	public function slice($slice) {
		return $this->operator('$slice', $slice);
	}


	public function sort($fieldName, $order = null) {
		$sort = array();
		$fields = is_array($fieldName) ? $fieldName : array($fieldName => $order);

		foreach ($fields as $fieldName => $order) {
			if (is_string($order)) {
				$order = strtolower($order) === 'asc' ? 1 : -1;
			}
			$sort[$fieldName] = (integer)$order;
		}

		return $this->operator('$sort', $sort);
	}


	public function where($javascript) {
		$this->query['$where'] = $javascript;
		return $this;
	}

	private function wrapEqualityCriteria() {
		/* If the current field has no criteria yet, do nothing. This ensures
		 * that we do not inadvertently inject {"$in": null} into the query.
		 */
		if ($this->currentField && !isset($this->query[$this->currentField]) && !array_key_exists($this->currentField, $this->query)) {
			return;
		}

		if ($this->currentField) {
			$query = &$this->query[$this->currentField];
		} else {
			$query = &$this->query;
		}

		/* If the query is an empty array, we'll assume that the user has not
		 * specified criteria. Otherwise, check if the array includes a query
		 * operator (checking the first key is sufficient). If neither of these
		 * conditions are met, we'll wrap the query value with $in.
		 */
		if (is_array($query) && (empty($query) || strpos(key($query), '$') === 0)) {
			return;
		}

		$query = array('$in' => array($query));
	}
}
