/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: PREV_CONS_LISTA

Descrio:
Clculo da Previso de Consumo atravs da stored procedure
PREVISAO_DE_CONSUMO_PLA, modificada para calcular para vrias filiais
(lista).

@P_NRLISTAFILI  - Nmero da Lista de Filiais e Servios Concatenados
@P_DTINICIAL    - Data inicial de clculo
@P_DTFINAL      - Data final de clculo
@P_CDPRODUTOINI - Cdigo do produto inicial (CDARVPROD)
@P_CDPRODUTOFIN - Cdigo do produto final   (CDARVPROD)
@P_PEACRESCIMO  - Percentual de acrscimo
@P_IDARREDOND   - Opo de arredondamento.
                 Formato:
                    S - ARREDONDAMENTO DE REQUISIO
                    N - NAO ARREDONDAR
                    C - ARREDONDAMENTO DE COMPRAS
@P_CDOPERADOR   - Cdigo do operador que solicitou o clculo
@P_DTHRCALCPC   - Data/hora do clculo

EXEC PREV_CONS_LISTA '0000038266', '01/05/2005', '31/05/2005', '0', 'Z', 0, 'N', '000000000999', GETDATE()

Criao:
23/03/06 - Dmaso (p/ GR - v. 4.06)
25/05/2006 - Leonardo Leo - Converso para SQLSERVER

*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'PREV_CONS_LISTA' AND
                 TYPE = 'P'    AND
                 U.UID = O.UID AND
                 U.NAME = USER)
   DROP PROCEDURE PREV_CONS_LISTA
GO

CREATE PROCEDURE PREV_CONS_LISTA
  (@P_NRLISTAFILI  VARCHAR(10),
   @P_DTINICIAL    DATETIME,
   @P_DTFINAL      DATETIME,
   @P_CDPRODUTOINI VARCHAR(10),
   @P_CDPRODUTOFIN VARCHAR(10),
   @P_PEACRESCIMO  NUMERIC(15,6),
   @P_IDARREDOND   VARCHAR(1),
   @P_CDOPERADOR   VARCHAR(12),
   @P_DTHRCALCPC   DATETIME)
AS
   DECLARE @V_PRXCONSPLAN VARCHAR(20);
   DECLARE @V_CDFILIAL    VARCHAR(4);
   DECLARE @V_CDSERVICO   VARCHAR(5);
   DECLARE @V_LCDSERVICO  VARCHAR(4000);
   DECLARE @ERRO INT;
BEGIN
   DECLARE CUR_FILIAL CURSOR FOR
      SELECT DISTINCT
         SUBSTRING(CDLISTA, 1, 4)
      FROM
         LISTA
      WHERE
         NRLISTA = @P_NRLISTAFILI;
   OPEN CUR_FILIAL;

   FETCH NEXT FROM CUR_FILIAL INTO @V_CDFILIAL;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SET @V_LCDSERVICO = '_';

      DECLARE CUR_CDSERVICO CURSOR FOR
         SELECT
            S.CDSERVFILI
         FROM
            SERVICO S,
            LISTA L
         WHERE
            L.NRLISTA  = @P_NRLISTAFILI AND
            S.CDFILIAL = @V_CDFILIAL    AND
            L.CDLISTA  = S.CDFILIAL + S.CDSERVFILI;
      OPEN CUR_CDSERVICO;
      FETCH NEXT FROM CUR_CDSERVICO INTO @V_CDSERVICO;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         SET @V_LCDSERVICO = @V_LCDSERVICO + @V_CDSERVICO + '_';
         FETCH NEXT FROM CUR_CDSERVICO INTO @V_CDSERVICO;
      END;
      CLOSE CUR_CDSERVICO;
      DEALLOCATE CUR_CDSERVICO;

      SET @V_PRXCONSPLAN = '';
      EXEC NOVO_CODIGO 'CONSPLAN', @P_SEQUENCIAL = @V_PRXCONSPLAN OUTPUT, @P_QTDE = 1;
      SET @V_PRXCONSPLAN = SUBSTRING(@V_PRXCONSPLAN, 11, 10);

      EXEC @ERRO = PREVISAO_DE_CONSUMO_PLA @V_CDFILIAL,
                                           @V_LCDSERVICO,
                                           @P_DTINICIAL,
                                           @P_DTFINAL,
                                           @P_PEACRESCIMO,
                                           @P_CDPRODUTOINI,
                                           @P_CDPRODUTOFIN,
                                           @P_CDOPERADOR,
                                           @P_DTHRCALCPC,
                                           @V_PRXCONSPLAN,
                                           @P_IDARREDOND,
                                           NULL,
                                           NULL;

      IF @ERRO <> 0 BEGIN
         RETURN(1);
      END; 

      FETCH NEXT FROM CUR_FILIAL INTO @V_CDFILIAL;
   END;
   CLOSE CUR_FILIAL;
   DEALLOCATE CUR_FILIAL;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'PREV_CONS_LISTA','4.11.033';
GO

